package main

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"

	"messenger/message"
	"messenger/server"
	"messenger/user"
)

const SUBTIME_SAMPLE = 110
const NOTSUBTIME_SAMPLE = 290

func Test1(t *testing.T) {
	users := []*user.User{}
	for i := 0; i < 3; i++ {
		sub := true
		if i%2 == 1 {
			sub = false
		}
		u := user.NewUser(i, sub)
		users = append(users, u)
	}
	server.SetupServer(users, nil)

	users[0].SendMessage(1, "hi")
	users[1].SendMessage(2, "hello")

	time.Sleep(180 * time.Millisecond)

	msg := users[1].ReceiveMessage()
	assert.Equal(t, "hi", msg.Text)
	assert.Equal(t, SUBTIME_SAMPLE*time.Millisecond, msg.MsInQueueBeforeBeingSent)
	msg = users[2].ReceiveMessage()
	assert.Equal(t, message.Message{}, msg)

	time.Sleep(180 * time.Millisecond)

	msg = users[2].ReceiveMessage()
	assert.Equal(t, "hello", msg.Text)
	assert.Equal(t, NOTSUBTIME_SAMPLE*time.Millisecond, msg.MsInQueueBeforeBeingSent)
}

func Test2(t *testing.T) {
	users := []*user.User{}
	for i := 0; i < 5; i++ {
		sub := true
		if i%2 == 1 {
			sub = false
		}
		u := user.NewUser(i, sub)
		users = append(users, u)
	}
	grp5 := user.NewGroup(5, users[0:3]) //0 1 2
	groups := []*user.Group{grp5}
	server.SetupServer(users, groups)

	users[1].SendMessage(5, "hi")

	time.Sleep(100 * time.Millisecond)
	grp5.AddUser(users[3])

	time.Sleep(300 * time.Millisecond)
	msg := users[2].ReceiveMessage()
	assert.Equal(t, "hi", msg.Text)
	assert.Equal(t, NOTSUBTIME_SAMPLE*time.Millisecond, msg.MsInQueueBeforeBeingSent)
	msg = users[3].ReceiveMessage()
	assert.Equal(t, message.Message{}, msg)
}
